@@ -1,5 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+from ..common import _reproject_dispatcher
 from ..utils import parse_input_data, parse_output_projection
 from .core import _reproject_adaptive_2d
 
@@ -11,8 +12,8 @@ def reproject_adaptive(
     output_projection,
     shape_out=None,
     hdu_in=0,
-    return_footprint=True,
     center_jacobian=False,
+    despike_jacobian=False,
     roundtrip_coords=True,
     conserve_flux=False,
     kernel="gaussian",
@@ -23,6 +24,14 @@ def reproject_adaptive(
     boundary_ignore_threshold=0.5,
     x_cyclic=False,
     y_cyclic=False,
+    bad_value_mode="strict",
+    bad_fill_value=0,
+    output_array=None,
+    output_footprint=None,
+    return_footprint=True,
+    block_size=None,
+    parallel=False,
+    return_type=None,
 ):
     """
     Reproject a 2D array from one WCS to another using the DeForest (2004)
@@ -33,7 +42,7 @@ def reproject_adaptive(
 
     Parameters
     ----------
-    input_data
+    input_data : object
         The input data to reproject. This can be:
 
             * The name of a FITS file as a `str` or a `pathlib.Path` object
@@ -64,8 +73,6 @@ def reproject_adaptive(
     hdu_in : int or str, optional
         If ``input_data`` is a FITS file or an `~astropy.io.fits.HDUList`
         instance, specifies the HDU to use.
-    return_footprint : bool
-        Whether to return the footprint in addition to the output array.
     center_jacobian : bool
         A Jacobian matrix is calculated, representing
         d(input image coordinate) / d(output image coordinate),
@@ -82,6 +89,22 @@ def reproject_adaptive(
         efficient, and the loss of accuracy is extremely small for
         transformations that vary smoothly between pixels. Defaults to
         ``False``.
+    despike_jacobian : bool
+        Whether to despike the computed Jacobian values. In some situations
+        (e.g. an all-sky map, with a wrap point in the longitude), extremely
+        large Jacobian values may be computed which are artifacts of the
+        coordinate system definition, rather than reflecting the actual nature
+        of the coordinate transformation. This may result in a band of ``nan``
+        pixels in the output image. In these situations, if the actual
+        transformation is approximately constant in the region of these
+        artifacts, this option should be enabled. If enabled, the typical
+        magnitude (distance from the determinant) of the Jacobian matrix,
+        ``Jmag2 = sum_j sum_i (J_ij**2)``, is computed for each pixel and
+        compared to the 25th percentile of that value in the local 3x3
+        neighborhood (i.e. the third-lowest value). If it exceeds that
+        percentile value by more than 10 times, the Jacobian matrix is deemed
+        to be "spiking" and it is replaced by the average of the non-spiking
+        values in the 3x3 neighborhood.
     roundtrip_coords : bool
         Whether to verify that coordinate transformations are defined in both
         directions.
@@ -91,9 +114,8 @@ def reproject_adaptive(
         The averaging kernel to use. Allowed values are 'Hann' and 'Gaussian'.
         Case-insensitive. The Gaussian kernel produces better photometric
         accuracy and stronger anti-aliasing at the cost of some blurring (on
-        the scale of a few pixels). If not specified, the Hann kernel is used
-        by default, but this will change to the Gaussian kernel in a future
-        release.
+        the scale of a few pixels). If not specified, the Gaussain kernel is
+        used by default.
     kernel_width : double
         The width of the kernel in pixels, measuring to +/- 1 sigma for the
         Gaussian window. Does not apply to the Hann window. Reducing this width
@@ -114,8 +136,8 @@ def reproject_adaptive(
         accuracy.
     boundary_mode : str
         How to handle when the sampling region includes regions outside the
-        bounds of the input image. The default is ``ignore``, but this will
-        change to ``strict`` in a future release. Allowed values are:
+        bounds of the input image. The default is ``strict``. Allowed values
+        are:
 
             * ``strict`` --- Output pixels will be NaN if any input sample
               falls outside the input image.
@@ -135,7 +157,7 @@ def reproject_adaptive(
               ``boundary_ignore_threshold`` argument. In that case, acts as
               ``strict``.
             * ``nearest`` --- Samples outside the input image are replaced by
-              the nearst in-bounds input pixel.
+              the nearest in-bounds input pixel.
 
     boundary_fill_value : double
         The constant value used by the ``constant`` boundary mode.
@@ -147,11 +169,51 @@ def reproject_adaptive(
         Indicates that the x or y axis of the input image should be treated as
         cyclic or periodic. Overrides the boundary mode for that axis, so that
         out-of-bounds samples wrap to the other side of the image.
+    bad_value_mode : str
+        How to handle values of ``nan`` and ``inf`` in the input data. The
+        default is ``strct``. Allowed values are:
+
+            * ``strict`` --- Values of ``nan`` or ``inf`` in the input data are
+              propagated to every output value which samples them.
+            * ``ignore`` --- When a sampled input value is ``nan`` or ``inf``,
+              that input pixel is ignored (affected neither the accumulated sum
+              of weighted samples nor the accumulated sum of weights).
+            * ``constant`` --- Input values of ``nan`` and ``inf`` are replaced
+              with a constant value, set via the ``bad_fill_value`` argument.
+
+    bad_fill_value : double
+        The constant value used by the ``constant`` bad-value mode.
+    output_array : None or `~numpy.ndarray`
+        An array in which to store the reprojected data.  This can be any numpy
+        array including a memory map, which may be helpful when dealing with
+        extremely large files.
+    output_footprint : `~numpy.ndarray`, optional
+        An array in which to store the footprint of reprojected data.  This can be
+        any numpy array including a memory map, which may be helpful when dealing with
+        extremely large files.
+    return_footprint : bool
+        Whether to return the footprint in addition to the output array.
+    block_size : tuple or 'auto', optional
+        The size of blocks in terms of output array pixels that each block will handle
+        reprojecting. Extending out from (0,0) coords positively, block sizes
+        are clamped to output space edges when a block would extend past edge.
+        Specifying ``'auto'`` means that reprojection will be done in blocks with
+        the block size automatically determined. If ``block_size`` is not
+        specified or set to `None`, the reprojection will not be carried out in
+        blocks.
+    parallel : bool or int, optional
+        If `True`, the reprojection is carried out in parallel, and if a
+        positive integer, this specifies the number of processes to use.
+        The reprojection will be parallelized over output array blocks specified
+        by ``block_size`` (if the block size is not set, it will be determined
+        automatically).
+    return_type : {'numpy', 'dask'}, optional
+        Whether to return numpy or dask arrays - defaults to 'numpy'.
 
     Returns
     -------
     array_new : `~numpy.ndarray`
-        The reprojected array
+        The reprojected array.
     footprint : `~numpy.ndarray`
         Footprint of the input array in the output array. Values of 0 indicate
         no coverage or valid values in the input image, while values of 1
@@ -165,21 +227,32 @@ def reproject_adaptive(
         output_projection, shape_in=array_in.shape, shape_out=shape_out
     )
 
-    return _reproject_adaptive_2d(
-        array_in,
-        wcs_in,
-        wcs_out,
-        shape_out,
+    return _reproject_dispatcher(
+        _reproject_adaptive_2d,
+        array_in=array_in,
+        wcs_in=wcs_in,
+        wcs_out=wcs_out,
+        shape_out=shape_out,
+        array_out=output_array,
+        parallel=parallel,
+        block_size=block_size,
         return_footprint=return_footprint,
-        center_jacobian=center_jacobian,
-        roundtrip_coords=roundtrip_coords,
-        conserve_flux=conserve_flux,
-        kernel=kernel,
-        kernel_width=kernel_width,
-        sample_region_width=sample_region_width,
-        boundary_mode=boundary_mode,
-        boundary_fill_value=boundary_fill_value,
-        boundary_ignore_threshold=boundary_ignore_threshold,
-        x_cyclic=x_cyclic,
-        y_cyclic=y_cyclic,
+        output_footprint=output_footprint,
+        reproject_func_kwargs=dict(
+            center_jacobian=center_jacobian,
+            despike_jacobian=despike_jacobian,
+            roundtrip_coords=roundtrip_coords,
+            conserve_flux=conserve_flux,
+            kernel=kernel,
+            kernel_width=kernel_width,
+            sample_region_width=sample_region_width,
+            boundary_mode=boundary_mode,
+            boundary_fill_value=boundary_fill_value,
+            boundary_ignore_threshold=boundary_ignore_threshold,
+            x_cyclic=x_cyclic,
+            y_cyclic=y_cyclic,
+            bad_value_mode=bad_value_mode,
+            bad_fill_value=bad_fill_value,
+        ),
+        return_type=return_type,
     )
