@@ -66,6 +66,69 @@ def test_reproject_adaptive_2d(wcsapi, center_jacobian, roundtrip_coords):
     return array_footprint_to_hdulist(array_out, footprint_out, header_out)
 
 
+@pytest.mark.parametrize("axis", ("x", "y"))
+@pytest.mark.parametrize("center_jacobian", (True, False))
+def test_reproject_adaptive_despike_jacobian(axis, center_jacobian):
+    if axis == "x":
+        wcs_in = WCS(naxis=2)
+        wcs_in.wcs.ctype = "RA---CAR", "DEC--CAR"
+        wcs_in.wcs.crpix = 18.5, 9.5
+        wcs_in.wcs.crval = 180, 0
+        wcs_in.wcs.cdelt = 10, 10
+        data_in = np.arange(18 * 36).reshape((18, 36))
+
+        wcs_out = WCS(naxis=2)
+        wcs_out.wcs.ctype = "RA---CAR", "DEC--CAR"
+        wcs_out.wcs.crpix = 20, 20
+        wcs_out.wcs.crval = 10, 0
+        wcs_out.wcs.cdelt = 5, 1
+    else:
+        wcs_in = WCS(naxis=2)
+        wcs_in.wcs.ctype = "DEC--CAR", "RA---CAR"
+        wcs_in.wcs.crpix = 9.5, 18.5
+        wcs_in.wcs.crval = 0, 180
+        wcs_in.wcs.cdelt = 10, 10
+        data_in = np.arange(18 * 36).reshape((36, 18))
+
+        wcs_out = WCS(naxis=2)
+        wcs_out.wcs.ctype = "DEC--CAR", "RA---CAR"
+        wcs_out.wcs.crpix = 20, 20
+        wcs_out.wcs.crval = 0, 10
+        wcs_out.wcs.cdelt = 1, 5
+
+    output = reproject_adaptive(
+        (data_in, wcs_in),
+        wcs_out,
+        (40, 40),
+        roundtrip_coords=False,
+        return_footprint=False,
+        despike_jacobian=False,
+        center_jacobian=center_jacobian,
+        x_cyclic=(axis == "x"),
+        y_cyclic=(axis == "y"),
+    )
+
+    output_despiked = reproject_adaptive(
+        (data_in, wcs_in),
+        wcs_out,
+        (40, 40),
+        roundtrip_coords=False,
+        return_footprint=False,
+        despike_jacobian=True,
+        center_jacobian=center_jacobian,
+        x_cyclic=(axis == "x"),
+        y_cyclic=(axis == "y"),
+    )
+
+    finite = np.isfinite(output)
+    # There should be a strip of nans in the un-despiked output
+    assert np.sum(finite) < output.size
+    # The outputs should match where spiking wasn't a concern
+    np.testing.assert_equal(output[finite], output_despiked[finite])
+    # The despiked output should have no nans
+    assert np.all(np.isfinite(output_despiked))
+
+
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.array_compare(single_reference=True)
 @pytest.mark.parametrize("center_jacobian", (False, True))
@@ -616,6 +679,90 @@ def test_boundary_modes(x_cyclic, y_cyclic, rotated):
     assert not np.any(np.isnan(data_out))
 
 
+@pytest.mark.parametrize("bad_val", (np.nan, np.inf))
+def test_bad_value_modes(bad_val):
+    bad_val_checker = np.isnan if np.isnan(bad_val) else np.isinf
+    data_in = np.ones((30, 30))
+    data_in[15, 15] = bad_val
+
+    wcs_in = WCS(naxis=2)
+    wcs_in.wcs.crpix = [15.5, 15.5]
+    wcs_in.wcs.crval = [0, 0]
+    wcs_in.wcs.cdelt = [1, 1]
+
+    # Our reprojection will be a no-op, so it's easy to reason about what
+    # should happen
+    wcs_out = wcs_in.deepcopy()
+
+    data_out = reproject_adaptive(
+        (data_in, wcs_in),
+        wcs_out,
+        shape_out=(30, 30),
+        return_footprint=False,
+        boundary_mode="ignore",
+        sample_region_width=3,
+        bad_value_mode="strict",
+    )
+
+    # With a sample_region_width of 3, we expect a 3x3 box of nans centered on
+    # the input-image nan.
+    assert np.all(bad_val_checker(data_out[14:17, 14:17]))
+    # And they should be the only nans
+    assert np.sum(bad_val_checker(data_out)) == 9
+
+    data_out = reproject_adaptive(
+        (data_in, wcs_in),
+        wcs_out,
+        shape_out=(30, 30),
+        return_footprint=False,
+        boundary_mode="ignore",
+        sample_region_width=3,
+        bad_value_mode="constant",
+        bad_fill_value=10,
+    )
+
+    # Now, where there were nans, we should get values > 1, since 10 is our
+    # fill value.
+    assert np.all(data_out[14:17, 14:17] > 1)
+    data_out[14:17, 14:17] = 1
+    np.testing.assert_equal(data_out, 1)
+
+    data_out = reproject_adaptive(
+        (data_in, wcs_in),
+        wcs_out,
+        shape_out=(30, 30),
+        return_footprint=False,
+        boundary_mode="ignore",
+        sample_region_width=3,
+        bad_value_mode="ignore",
+    )
+
+    # Now the nan should be ignored and we should only get 1s coming out
+    np.testing.assert_equal(data_out, 1)
+
+
+def test_invald_bad_value_mode():
+    data_in = np.ones((30, 30))
+
+    wcs_in = WCS(naxis=2)
+    wcs_in.wcs.crpix = [15.5, 15.5]
+    wcs_in.wcs.crval = [0, 0]
+    wcs_in.wcs.cdelt = [1, 1]
+
+    wcs_out = wcs_in.deepcopy()
+
+    with pytest.raises(ValueError, match="bad_value_mode 'invalid_mode' not recognized"):
+        reproject_adaptive(
+            (data_in, wcs_in),
+            wcs_out,
+            shape_out=(30, 30),
+            return_footprint=False,
+            boundary_mode="ignore",
+            sample_region_width=3,
+            bad_value_mode="invalid_mode",
+        )
+
+
 def prepare_test_data(file_format):
     pytest.importorskip("sunpy", minversion="2.1.0")
     from sunpy.coordinates.ephemeris import get_body_heliographic_stonyhurst
@@ -694,7 +841,11 @@ def test_reproject_adaptive_uncentered_jacobian():
 
 
 def _setup_for_broadcast_test(
-    conserve_flux=False, boundary_mode="strict", kernel="gaussian", center_jacobian=False
+    conserve_flux=False,
+    boundary_mode="strict",
+    kernel="gaussian",
+    center_jacobian=False,
+    bad_value_mode="strict",
 ):
     with fits.open(get_pkg_data_filename("data/galactic_2d.fits", package="reproject.tests")) as pf:
         hdu_in = pf[0]
@@ -709,6 +860,10 @@ def _setup_for_broadcast_test(
 
     image_stack = np.stack((data, data.T, data[::-1], data[:, ::-1]))
 
+    # Ensure we exercise the bad-value handling modes
+    image_stack[1, 30, 30] = np.nan
+    image_stack[2, 20, 40] = np.inf
+
     # Build the reference array through un-broadcast reprojections
     array_ref = np.empty_like(image_stack)
     footprint_ref = np.empty_like(image_stack)
@@ -720,6 +875,7 @@ def _setup_for_broadcast_test(
             boundary_mode=boundary_mode,
             kernel=kernel,
             center_jacobian=center_jacobian,
+            bad_value_mode=bad_value_mode,
         )
         array_ref[i] = array_out
         footprint_ref[i] = footprint_out
@@ -770,13 +926,18 @@ def test_broadcast_reprojection(input_extra_dims, output_shape, input_as_wcs, ou
 
 
 def _test_broadcast_reprojection_algo_specific_options(
-    conserve_flux=False, boundary_mode="strict", kernel="gaussian", center_jacobian=False
+    conserve_flux=False,
+    boundary_mode="strict",
+    kernel="gaussian",
+    center_jacobian=False,
+    bad_value_mode="strict",
 ):
     image_stack, array_ref, footprint_ref, header_in, header_out = _setup_for_broadcast_test(
         conserve_flux=conserve_flux,
         boundary_mode=boundary_mode,
         kernel=kernel,
         center_jacobian=center_jacobian,
+        bad_value_mode=bad_value_mode,
     )
 
     array_broadcast, footprint_broadcast = reproject_adaptive(
@@ -787,6 +948,7 @@ def _test_broadcast_reprojection_algo_specific_options(
         boundary_mode=boundary_mode,
         kernel=kernel,
         center_jacobian=center_jacobian,
+        bad_value_mode=bad_value_mode,
     )
 
     np.testing.assert_array_equal(footprint_broadcast, footprint_ref)
@@ -816,6 +978,11 @@ def test_broadcast_reprojection_center_jacobian(center_jacobian):
     _test_broadcast_reprojection_algo_specific_options(center_jacobian=center_jacobian)
 
 
+@pytest.mark.parametrize("bad_value_mode", ("strict", "ignore", "constant"))
+def test_broadcast_reprojection_bad_value_mode(bad_value_mode):
+    _test_broadcast_reprojection_algo_specific_options(bad_value_mode=bad_value_mode)
+
+
 def test_adaptive_input_output_types(
     valid_celestial_input_data, valid_celestial_output_projections
 ):
