@@ -17,11 +17,13 @@ def reproject_and_coadd(
     shape_out=None,
     input_weights=None,
     hdu_in=None,
-    reproject_function=None,
     hdu_weights=None,
+    reproject_function=None,
     combine_function="mean",
     match_background=False,
     background_reference=None,
+    output_array=None,
+    output_footprint=None,
     **kwargs,
 ):
     """
@@ -72,30 +74,48 @@ def reproject_and_coadd(
         If one or more items in ``input_weights`` is a FITS file or an
         `~astropy.io.fits.HDUList` instance, specifies the HDU to use.
     reproject_function : callable
-        The function to use for the reprojection
-    combine_function : { 'mean', 'sum', 'median' }
+        The function to use for the reprojection.
+    combine_function : { 'mean', 'sum', 'median', 'first', 'last', 'min', 'max' }
         The type of function to use for combining the values into the final
-        image.
+        image. For 'first' and 'last', respectively, the reprojected images are
+        simply overlaid on top of each other. With respect to the order of the
+        input images in ``input_data``, either the first or the last image to
+        cover a region of overlap determines the output data for that region.
     match_background : bool
         Whether to match the backgrounds of the images.
     background_reference : `None` or `int`
         If `None`, the background matching will make it so that the average of
         the corrections for all images is zero. If an integer, this specifies
         the index of the image to use as a reference.
-    kwargs
+    output_array : array or None
+        The final output array.  Specify this if you already have an
+        appropriately-shaped array to store the data in.  Must match shape
+        specified with ``shape_out`` or derived from the output projection.
+    output_footprint : array or None
+        The final output footprint array.  Specify this if you already have an
+        appropriately-shaped array to store the data in.  Must match shape
+        specified with ``shape_out`` or derived from the output projection.
+    **kwargs
         Keyword arguments to be passed to the reprojection function.
+
+    Returns
+    -------
+    array : `~numpy.ndarray`
+        The co-added array.
+    footprint : `~numpy.ndarray`
+        Footprint of the co-added array. Values of 0 indicate no coverage or
+        valid values in the input image, while values of 1 indicate valid
+        values.
     """
 
     # TODO: add support for saving intermediate files to disk to avoid blowing
     # up memory usage. We could probably still have references to array
     # objects, but we'd just make sure these were memory mapped
 
-    # TODO: add support for specifying output array
-
     # Validate inputs
 
-    if combine_function not in ("mean", "sum", "median"):
-        raise ValueError("combine_function should be one of mean/sum/median")
+    if combine_function not in ("mean", "sum", "median", "first", "last", "min", "max"):
+        raise ValueError("combine_function should be one of mean/sum/median/first/last/min/max")
 
     if reproject_function is None:
         raise ValueError(
@@ -106,6 +126,17 @@ def reproject_and_coadd(
 
     wcs_out, shape_out = parse_output_projection(output_projection, shape_out=shape_out)
 
+    if output_array is not None and output_array.shape != shape_out:
+        raise ValueError(
+            "If you specify an output array, it must have a shape matching "
+            f"the output shape {shape_out}"
+        )
+    if output_footprint is not None and output_footprint.shape != shape_out:
+        raise ValueError(
+            "If you specify an output footprint array, it must have a shape matching "
+            f"the output shape {shape_out}"
+        )
+
     # Start off by reprojecting individual images to the final projection
 
     arrays = []
@@ -125,16 +156,21 @@ def reproject_and_coadd(
 
         # Since we might be reprojecting small images into a large mosaic we
         # want to make sure that for each image we reproject to an array with
-        # minimal footprint. We therefore find the pixel coordinates of corners
-        # in the initial image and transform this to pixel coordinates in the
-        # final image to figure out the final WCS and shape to reproject to for
-        # each tile. Note that in future if we are worried about significant
-        # distortions of the edges in the reprojection process we could simply
-        # add arbitrary numbers of midpoints to this list.
+        # minimal footprint. We therefore find the pixel coordinates of the
+        # edges of the initial image and transform this to pixel coordinates in
+        # the final image to figure out the final WCS and shape to reproject to
+        # for each tile. We strike a balance between transforming only the
+        # input-image corners, which is fast but can cause clipping in cases of
+        # significant distortion (when the edges of the input image become
+        # convex in the output projection), and transforming every edge pixel,
+        # which provides a lot of redundant information.
         ny, nx = array_in.shape
-        xc = np.array([-0.5, nx - 0.5, nx - 0.5, -0.5])
-        yc = np.array([-0.5, -0.5, ny - 0.5, ny - 0.5])
-        xc_out, yc_out = wcs_out.world_to_pixel(wcs_in.pixel_to_world(xc, yc))
+        n_per_edge = 11
+        xs = np.linspace(-0.5, nx - 0.5, n_per_edge)
+        ys = np.linspace(-0.5, ny - 0.5, n_per_edge)
+        xs = np.concatenate((xs, np.full(n_per_edge, xs[-1]), xs, np.full(n_per_edge, xs[0])))
+        ys = np.concatenate((np.full(n_per_edge, ys[0]), ys, np.full(n_per_edge, ys[-1]), ys))
+        xc_out, yc_out = wcs_out.world_to_pixel(wcs_in.pixel_to_world(xs, ys))
 
         # Determine the cutout parameters
 
@@ -215,10 +251,15 @@ def reproject_and_coadd(
 
     # At this point, the images are now ready to be co-added.
 
-    # TODO: provide control over final dtype
+    if output_array is None:
+        output_array = np.zeros(shape_out)
+    if output_footprint is None:
+        output_footprint = np.zeros(shape_out)
 
-    final_array = np.zeros(shape_out)
-    final_footprint = np.zeros(shape_out)
+    if combine_function == "min":
+        output_array[...] = np.inf
+    elif combine_function == "max":
+        output_array[...] = -np.inf
 
     if combine_function in ("mean", "sum"):
         for array in arrays:
@@ -227,12 +268,34 @@ def reproject_and_coadd(
             # means/sums.
             array.array[array.footprint == 0] = 0
 
-            final_array[array.view_in_original_array] += array.array * array.footprint
-            final_footprint[array.view_in_original_array] += array.footprint
+            output_array[array.view_in_original_array] += array.array * array.footprint
+            output_footprint[array.view_in_original_array] += array.footprint
 
         if combine_function == "mean":
             with np.errstate(invalid="ignore"):
-                final_array /= final_footprint
+                output_array /= output_footprint
+
+    elif combine_function in ("first", "last", "min", "max"):
+        for array in arrays:
+            if combine_function == "first":
+                mask = output_footprint[array.view_in_original_array] == 0
+            elif combine_function == "last":
+                mask = array.footprint > 0
+            elif combine_function == "min":
+                mask = (array.footprint > 0) & (
+                    array.array < output_array[array.view_in_original_array]
+                )
+            elif combine_function == "max":
+                mask = (array.footprint > 0) & (
+                    array.array > output_array[array.view_in_original_array]
+                )
+
+            output_footprint[array.view_in_original_array] = np.where(
+                mask, array.footprint, output_footprint[array.view_in_original_array]
+            )
+            output_array[array.view_in_original_array] = np.where(
+                mask, array.array, output_array[array.view_in_original_array]
+            )
 
     elif combine_function == "median":
         # Here we need to operate in chunks since we could otherwise run
@@ -240,4 +303,7 @@ def reproject_and_coadd(
 
         raise NotImplementedError("combine_function='median' is not yet implemented")
 
-    return final_array, final_footprint
+    if combine_function in ("min", "max"):
+        output_array[output_footprint == 0] = 0.0
+
+    return output_array, output_footprint
