@@ -41,11 +41,10 @@ class TestReprojectAndCoAdd:
 
         input_data = []
 
-        for jmin, jmax, imin, imax in views:
-            array = self.array[jmin:jmax, imin:imax].copy()
+        for view in views:
+            array = self.array[view].copy()
             wcs = self.wcs.deepcopy()
-            wcs.wcs.crpix[0] -= imin
-            wcs.wcs.crpix[1] -= jmin
+            wcs = wcs[view]
             input_data.append((array, wcs))
 
         return input_data
@@ -58,7 +57,7 @@ class TestReprojectAndCoAdd:
         views = []
         for i in range(4):
             for j in range(5):
-                views.append((je[j], je[j + 1], ie[i], ie[i + 1]))
+                views.append(np.s_[je[j] : je[j + 1], ie[i] : ie[i + 1]])
 
         return views
 
@@ -70,11 +69,11 @@ class TestReprojectAndCoAdd:
         views = []
         for i in range(4):
             for j in range(5):
-                views.append((je[j], je[j + 1] + 10, ie[i], ie[i + 1] + 10))
+                views.append(np.s_[je[j] : je[j + 1] + 10, ie[i] : ie[i + 1] + 10])
 
         return views
 
-    @pytest.mark.parametrize("combine_function", ["mean", "sum"])
+    @pytest.mark.parametrize("combine_function", ["mean", "sum", "first", "last"])
     def test_coadd_no_overlap(self, combine_function, reproject_function):
         # Make sure that if all tiles are exactly non-overlapping, and
         # we use 'sum' or 'mean', we get the exact input array back.
@@ -109,6 +108,72 @@ class TestReprojectAndCoAdd:
 
         assert_allclose(array, self.array, atol=ATOL)
 
+    def test_coadd_with_outputs(self, tmp_path, reproject_function):
+        # Test the options to specify output array/footprint
+
+        input_data = self._get_tiles(self._overlapping_views)
+
+        output_array = np.memmap(
+            tmp_path / "array.np", mode="w+", dtype=float, shape=self.array.shape
+        )
+        output_footprint = np.memmap(
+            tmp_path / "footprint.np", mode="w+", dtype=float, shape=self.array.shape
+        )
+
+        array, footprint = reproject_and_coadd(
+            input_data,
+            self.wcs,
+            shape_out=self.array.shape,
+            combine_function="mean",
+            reproject_function=reproject_function,
+            output_array=output_array,
+            output_footprint=output_footprint,
+        )
+
+        assert_allclose(output_array, self.array, atol=ATOL)
+        assert_allclose(output_footprint, footprint, atol=ATOL)
+
+    @pytest.mark.parametrize("combine_function", ["first", "last", "min", "max"])
+    def test_coadd_with_overlap_first_last(self, reproject_function, combine_function):
+        views = self._overlapping_views
+        input_data = self._get_tiles(views)
+
+        # Make each of the overlapping tiles different
+        for i, (array, wcs) in enumerate(input_data):
+            # We give each tile integer values that range from 0 to 19 but we
+            # deliberately don't make the first one 0 and the last one 19 so
+            # that min/max differs from first/last.
+            input_data[i] = (np.full_like(array, (i + 7) % 20), wcs)
+
+        array, footprint = reproject_and_coadd(
+            input_data,
+            self.wcs,
+            shape_out=self.array.shape,
+            combine_function=combine_function,
+            reproject_function=reproject_function,
+        )
+
+        # Test that either the correct tile sets the output value in the overlap regions
+        test_sequence = list(enumerate(views))
+
+        if combine_function == "last":
+            test_sequence = test_sequence[::-1]
+        elif combine_function == "min":
+            test_sequence = test_sequence[13:] + test_sequence[:13]
+        elif combine_function == "max":
+            test_sequence = (test_sequence[13:] + test_sequence[:13])[::-1]
+
+        for i, view in test_sequence:
+            # Each tile in test_sequence should overwrite the following tiles
+            # in the overlap regions. We'll use NaNs to mark pixels in the
+            # output array that have already been set by a preceding tile, so
+            # we'll go through, check that each tile matches the non-nan pixels
+            # in its region, and then set that whole region to nan.
+            output_tile = array[view]
+            output_values = output_tile[np.isfinite(output_tile)]
+            assert_allclose(output_values, (i + 7) % 20)
+            array[view] = np.nan
+
     def test_coadd_background_matching(self, reproject_function):
         # Test out the background matching
 
