@@ -71,12 +71,12 @@ def find_optimal_celestial_wcs(
         instance, specifies the HDU to use.
     frame : str or `~astropy.coordinates.BaseCoordinateFrame`
         The coordinate system for the final image (defaults to the frame of
-        the first image specified)
+        the first image specified).
     auto_rotate : bool
         Whether to rotate the header to minimize the final image area (if
-        `True`, requires shapely>=1.6 to be installed)
+        `True`, requires shapely>=1.6 to be installed).
     projection : str
-        Three-letter code for the WCS projection
+        Three-letter code for the WCS projection.
     resolution : `~astropy.units.Quantity`
         The resolution of the final image. If not specified, this is the
         smallest resolution of the input images.
