@@ -1,6 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-import signal
 import warnings
 
 import numpy as np
@@ -8,34 +7,23 @@ from astropy import units as u
 from astropy.wcs import WCS
 from astropy.wcs.utils import proj_plane_pixel_area
 
-
-def _init_worker():
-    """
-    Function to disable ctrl+c in the worker processes.
-    """
-    signal.signal(signal.SIGINT, signal.SIG_IGN)
+from ._overlap import _reproject_slice_cython
 
 
-def _reproject_slice(args):
-    from ._overlap import _reproject_slice_cython
+def _reproject_celestial(
+    array,
+    wcs_in,
+    wcs_out,
+    shape_out,
+    array_out=None,
+    output_footprint=None,
+    return_footprint=True,
+):
+    if array_out is None:
+        array_out = np.empty(shape_out)
 
-    return _reproject_slice_cython(*args)
-
-
-def _reproject_celestial(array, wcs_in, wcs_out, shape_out, parallel=True, return_footprint=True):
-    # Check the parallel flag.
-    if type(parallel) != bool and type(parallel) != int:
-        raise TypeError("The 'parallel' flag must be a boolean or integral value")
-
-    if type(parallel) == int:
-        # parallel is a number of processes.
-        if parallel <= 0:
-            raise ValueError("The number of processors to use must be strictly positive")
-        nproc = parallel
-    else:
-        # parallel is a boolean flag. nproc = None here means automatically selected
-        # number of processes.
-        nproc = None if parallel else 1
+    if output_footprint is None:
+        output_footprint = np.empty(shape_out)
 
     # There are currently precision issues below certain resolutions, so we
     # emit a warning if this is the case. For more details, see:
@@ -123,100 +111,57 @@ def _reproject_celestial(array, wcs_in, wcs_out, shape_out, parallel=True, retur
         # We don't need to broadcast the transformation over any extra
         # axes---add an extra axis of length one just so we have something
         # to loop over in all cases.
+        broadcasting = False
         array = array.reshape((1, *array.shape))
     elif len(array.shape) > wcs_in.low_level_wcs.pixel_n_dim:
         # We're broadcasting. Flatten the extra dimensions so there's just one
         # to loop over
+        broadcasting = True
         array = array.reshape((-1, *array.shape[-wcs_in.low_level_wcs.pixel_n_dim :]))
+        array_out = array_out.reshape((-1, *shape_out[-2:]))
+        output_footprint = output_footprint.reshape((-1, *shape_out[-2:]))
     else:
         raise ValueError("Too few dimensions for input array")
 
-    # Put together the parameters common both to the serial and parallel implementations. The aca
-    # function is needed to enforce that the array will be contiguous when passed to the low-level
-    # raw C function, otherwise Cython might complain.
+    array = np.ascontiguousarray(array)
 
-    aca = np.ascontiguousarray
-    common_func_par = [
-        0,
-        ny_in,
-        nx_out,
-        ny_out,
-        aca(xp_inout),
-        aca(yp_inout),
-        aca(xw_in),
-        aca(yw_in),
-        aca(xw_out),
-        aca(yw_out),
-        None,  # input data
-        shape_out[-2:],
-    ]
-    array = aca(array)
-
-    if nproc is None or nproc > 1:
-        # Spin up our process pool outside the loop over broadcast dimensions
-        from multiprocessing import Pool, cpu_count
-
-        # If needed, establish the number of processors to use.
-        if nproc is None:
-            nproc = cpu_count()
-
-        # Prime each process in the pool with a small function that disables
-        # the ctrl+c signal in the child process.
-        pool = Pool(nproc, _init_worker)
-
-    outputs = []
-    output_weights = []
     for i in range(len(array)):
-        common_func_par[-2] = array[i]
+        array_new, weights = _reproject_slice_cython(
+            0,
+            nx_in,
+            0,
+            ny_in,
+            nx_out,
+            ny_out,
+            np.ascontiguousarray(xp_inout),
+            np.ascontiguousarray(yp_inout),
+            np.ascontiguousarray(xw_in),
+            np.ascontiguousarray(yw_in),
+            np.ascontiguousarray(xw_out),
+            np.ascontiguousarray(yw_out),
+            array[i],
+            shape_out[-2:],
+        )
 
-        if nproc == 1:
-            array_new, weights = _reproject_slice([0, nx_in] + common_func_par)
+        with np.errstate(invalid="ignore"):
+            array_new /= weights
 
-            with np.errstate(invalid="ignore"):
-                array_new /= weights
-
-            outputs.append(array_new)
+        if broadcasting:
+            print(array_out.shape, array_new.shape)
+            array_out[i] = array_new
             if return_footprint:
-                output_weights.append(weights)
-
-        elif nproc > 1:
-            inputs = []
-            for i in range(nproc):
-                start = int(nx_in) // nproc * i
-                end = int(nx_in) if i == nproc - 1 else int(nx_in) // nproc * (i + 1)
-                inputs.append([start, end] + common_func_par)
-
-            results = pool.map(_reproject_slice, inputs)
-
-            array_new, weights = zip(*results)
-
-            array_new = sum(array_new)
-            weights = sum(weights)
-
-            with np.errstate(invalid="ignore"):
-                array_new /= weights
-
-            outputs.append(array_new)
+                output_footprint[i] = weights
+        else:
+            array_out[:] = array_new
             if return_footprint:
-                output_weights.append(weights)
+                output_footprint[:] = weights
 
-    if nproc > 1:
-        pool.close()
-
-    if len(shape_out) == wcs_out.low_level_wcs.pixel_n_dim:
-        # We weren't broadcasting, so don't return any extra dimensions
-        outputs = outputs[0]
+    if broadcasting:
+        array_out = array_out.reshape(shape_out)
         if return_footprint:
-            output_weights = output_weights[0]
-    else:
-        outputs = np.stack(outputs)
-        # If we're broadcasting over multiple dimensions, impose them here
-        outputs.shape = shape_out
-        if return_footprint:
-            output_weights = np.stack(output_weights)
-            output_weights.shape = shape_out
+            output_footprint = output_footprint.reshape(shape_out)
 
     if return_footprint:
-        return outputs, output_weights
+        return array_out, output_footprint
     else:
-        return outputs
+        return array_out
